﻿using System;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace pass2u
{
    class SDK
    {
        String API_KEY = "YOUR_API_KEY_HERE"; // get api key from Pass2U: contact@micromacro.com.tw
        String HOST = "https://api.pass2u.net";
        String HEADER_X_API_KEY = "x-api-key";

        // POST to 
        // https://api.pass2u.net/v2/models/{modelId}/passes?source={sourceId}
        public async Task CreatePass(Int32 modelId, String sourceId, String payload)
        {
            String url = HOST + String.Format("/v2/models/{0}/passes?source={1}", 
                    modelId, sourceId, payload);
            Console.WriteLine("===== CreatePass =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("sourceId: " + sourceId);
            Console.WriteLine("payload: " + payload);
            Console.WriteLine("endpoint url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            HttpContent content = new StringContent(payload);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var response = await client.PostAsync(url, content); // http post
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
            var resObject = JsonConvert.DeserializeObject<dynamic>(body);
            Console.WriteLine("passId: " + resObject["passId"]);
        }

        // PUT to 
        // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}
        public async Task UpdatePass(Int32 modelId, String passId, String payload) {
            String url = HOST + String.Format("/v2/models/{0}/passes/{1}", modelId, passId);
            Console.WriteLine("===== UpdatePass =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("passId: " + passId);
            Console.WriteLine("payload: " + payload);
            Console.WriteLine("endpoint url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            HttpContent content = new StringContent(payload);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var response = await client.PutAsync(url, content); // http put
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
            var resObject = JsonConvert.DeserializeObject<dynamic>(body);
            Console.WriteLine("updatedTime: " + resObject["updatedTime"]);
        }

        // DELETE to 
        // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/attributes/{attributeName}
        public async Task DeletePassAttribute(Int32 modelId, String passId, String attributeName) {
            String url = HOST + String.Format("/v2/models/{0}/passes/{1}/attributes/{2}", 
                    modelId, passId, attributeName);
            Console.WriteLine("===== DeletePassAttribute =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("passId: " + passId);
            Console.WriteLine("attributeName: " + attributeName);
            Console.WriteLine("url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            HttpContent content = new StringContent("");
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var response = await client.DeleteAsync(url); // http delete
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
        }

        // DELETE to
        // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/fields/{fieldKey}
        public async Task DeletePassCustomField(Int32 modelId, String passId, String fieldKey) {
            String url = HOST + String.Format("/v2/models/{0}/passes/{1}/fields/{2}", 
                    modelId, passId, fieldKey);
            Console.WriteLine("===== deletePassCustomField =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("passId: " + passId);
            Console.WriteLine("fieldKey: " + fieldKey);
            Console.WriteLine("url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            HttpContent content = new StringContent("");
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var response = await client.DeleteAsync(url); // http delete
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
        }

        // DELETE to 
        // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/attributes/{attributeName}
        public async Task DeleteFieldAttribute(Int32 modelId, String passId,  
                String fieldKey, String attributeName) {
            String url = HOST + String.Format("/v2/models/{0}/passes/{1}/fields/{2}/attributes/{3}", 
                    modelId, passId, fieldKey, attributeName);
            Console.WriteLine("===== deleteFieldAttribute =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("passId: " + passId);
            Console.WriteLine("fieldKey: " + fieldKey);
            Console.WriteLine("attributeName: " + attributeName);
            Console.WriteLine("url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            HttpContent content = new StringContent("");
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            var response = await client.DeleteAsync(url); // http delete
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
        }

        // GET to
        // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}
        public async Task GetPkPassFile(Int32 modelId, String passId, String output) {
            String url = HOST + String.Format("/v2/models/{0}/passes/{1}", modelId, passId);
            Console.WriteLine("===== getPkPassFile =====");
            Console.WriteLine("modelId: " + modelId);
            Console.WriteLine("passId: " + passId);
            Console.WriteLine("output: " + output);
            Console.WriteLine("url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/vnd.apple.pkpass"));
            var response = await client.GetAsync(url); // http get
            using (Stream body = await response.Content.ReadAsStreamAsync())
            {
                using (Stream outStream = File.Open(output, FileMode.Create))
                {
                    await body.CopyToAsync(outStream);
                }
            } 
            Console.WriteLine("Response code: " + response.StatusCode);
        }

        // POST to 
        // https://api.pass2u.net/v2/images
        public async Task UploadImage(String imageName) {
            String url = HOST + "/v2/images";
            Console.WriteLine("===== uploadImage =====");
            Console.WriteLine("imageName: " + imageName);
            Console.WriteLine("endpoint url: " + url);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add(HEADER_X_API_KEY, API_KEY);
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            ByteArrayContent content = new ByteArrayContent(File.ReadAllBytes(imageName));
            content.Headers.ContentType = new MediaTypeHeaderValue("image/png");
            var response = await client.PostAsync(url, content); // http post
            string body = await response.Content.ReadAsStringAsync();
            Console.WriteLine("Response code: " + response.StatusCode);
            Console.WriteLine("Response: " + body);
            var resObject = JsonConvert.DeserializeObject<dynamic>(body);
            Console.WriteLine("hex: " + resObject["hex"]);
        }
    }

    class Example
    {
        static Int32 TEST_MODEL_ID = YOUR_MODEL_ID; // YOUR_MODEL_ID HERE, ie. 2177
        static String TEST_PASS_ID = "YOUR_PASS_ID"; // YOUR_PASS_ID HERE, ie. "L7sST70pjsDO"
        static SDK sdk = new SDK();

        static async Task CreatePassExample() { // create a pass with all defaults.
            String payload = "{}";
            await sdk.CreatePass(TEST_MODEL_ID, "GOOGLE", payload);
        }

        static async Task CreatePassExample2() { // create a pass with custom fields
            // Two field keys are specified in your created model.
            String payload = @"{
                    ""fields"": [ {
                        ""key"": ""keyName"",
                        ""label"": ""Name"",
                        ""value"": ""Eric"",
                        ""changeMessage"": ""Name is %@""
                    },
                    {
                        ""key"": ""keyBirth"",
                        ""label"": ""Birth"",
                        ""value"": ""Jun 1"",
                        ""changeMessage"": ""Birth is %@""
                    }
                    ]
                }";
            await sdk.CreatePass(TEST_MODEL_ID, "GOOGLE", payload);
        }

        static async Task CreatePassExample3() {
            // Three field keys are specified in your created model.
            String payload = @"{
                    ""backgroundColor"": ""rgb(51, 50, 46)"",
                    ""foregroundColor"": ""rgb(237, 219, 21)"",
                    ""labelColor"": ""rgb(196, 54, 39)"",
                    ""description"": ""PetStore Membership card"",
                    ""organizationName"": ""PetStore Inc."",
                    ""logoText"": ""PetStore"",
                    ""voided"": false,
                    ""sharingProhibited"": true,
                    ""expirationDate"": ""2020-12-31T23:00:15+08:00"",
                    ""images"": [
                        {
                            ""type"": ""icon"",
                            ""hex"": ""1d0e5fe51895ae3409e54e46b141d832d1315dd7""
                        }],
                    ""barcode"": {
                        ""altText"": ""1234567890"",
                        ""message"": ""1234567890""
                        },
                    ""locations"":[
                        {
                            ""relevantText"": ""Store nearby on 1st and Main."",
                            ""latitude"": 25.0413093,
                            ""longitude"": 121.55269
                        },
                        {
                            ""relevantText"": ""Take a break with Coffee"",
                            ""latitude"": 12.041309,
                            ""longitude"": 81.322656
                        }],
                    ""beacons"":[
                        {
                            ""major"": 123,
                            ""minor"": 10,
                            ""proximityUUID"": ""B0792880-A495-A8AB-F738-081A98A572DE"",
                            ""relevantText"": ""PetStore lobby""
                        },
                        {
                            ""major"": 123,
                            ""minor"": 20,
                            ""proximityUUID"": ""B0792880-A495-A8AB-F738-081A98A572DE"",
                            ""relevantText"": ""PetStore garage""
                        }],
                    ""maxDistance"": 200,
                    ""relevantDate"": ""2020-10-18T16:53:00+08:00"",
                    ""fields"": [
                        {
                            ""key"": ""keyName"",
                            ""label"": ""Name"",
                            ""value"": ""Eric"",
                            ""changeMessage"": ""name is %@""
                        },
                        {
                            ""key"": ""keyBirth"",
                            ""label"": ""Birth"",
                            ""value"": ""Jun 1"",
                            ""changeMessage"": ""birth is %@""
                        },
                        {
                            ""key"": ""keyLevel"",
                            ""label"": ""Level"",
                            ""value"": ""GOLD"",
                            ""changeMessage"": ""level is %@""
                        }]                  
                }";
            //
            // groupingIdentifier are for both boarding pass and event ticket 
            // ""groupingIdentifier"": ""PetStore"", 
            //
            // transitType is for boarding pass only. can be one of following values: 
            // PKTransitTypeAir, PKTransitTypeBoat, PKTransitTypeBus, PKTransitTypeGeneric,PKTransitTypeTrain.
            // ""transitType"": ""PKTransitTypeAir"",
            //
            // Image types can be icon (at least), logo, strip, background, thumbnail, footer.
            // Uploaded image hex id is create by uploadImage()
            await sdk.CreatePass(TEST_MODEL_ID, "GOOGLE", payload);
        }

        static async Task UpdatePassExample() {
            // Basically, create pass and update pass has same payload structure
            // you can make subset of it.
            String payload = @"{
                ""backgroundColor"": ""rgb(151, 50, 46)"",
                ""foregroundColor"": ""rgb(137, 219, 21)"",
                ""labelColor"": ""rgb(96, 54, 39)"",
                ""description"": ""PetStore New Membership card"",
                ""organizationName"": ""PetStore Inc."",
                ""logoText"": ""PetStore"",
                ""voided"": true,
                ""sharingProhibited"": false,
                ""expirationDate"": ""2021-12-31T23:00:15+08:00"",
                ""images"": [
                    {
                        ""type"": ""strip"",
                        ""hex"": ""d33b9f1426d363340c2946557f173c402cd74014""
                    }],
                ""barcode"": {
                    ""altText"": ""0987654321"",
                    ""message"": ""0987654321""
                    },
                ""locations"":[
                    {
                        ""relevantText"": ""Store nearby on 1st and Main."",
                        ""latitude"": 25.0413093,
                        ""longitude"": 121.55269
                    }],
                ""beacons"":[
                    {
                        ""major"": 123,
                        ""minor"": 10,
                        ""proximityUUID"": ""B0792880-A495-A8AB-F738-081A98A572DE"",
                        ""relevantText"": ""PetStore lobby""
                    }],
                ""maxDistance"": 100,
                ""relevantDate"": ""2020-10-18T16:53:00+08:00"",
                ""fields"": [
                    {
                        ""key"": ""keyName"",
                        ""value"": ""Eric H"",
                        ""changeMessage"": ""name is %@""
                    },
                    {
                        ""key"": ""keyBirth"",
                        ""value"": ""July 1"",
                        ""changeMessage"": ""birth is %@""
                    }]
                }";
            await sdk.UpdatePass(TEST_MODEL_ID, TEST_PASS_ID, payload);
        }

        static async Task DeletePassAttributeExample() {
            String attributeToReset = "backgroundColor"; // tier-one pass attribute to reset          
            await sdk.DeletePassAttribute(TEST_MODEL_ID, TEST_PASS_ID, attributeToReset);
        }

        static async Task DeletePassCustomFieldExample() { 
            String attributeNameToDelete = "keyName"; // dynamic-type custom field key to delete        
            await sdk.DeletePassCustomField(TEST_MODEL_ID, TEST_PASS_ID, attributeNameToDelete);
        }

        static async Task DeleteFieldAttributeExample() {
            String fieldKey = "keyBirth"; // dynamic-type custom field key   
            // custom field attribute to reset. 
            // attribute can be one of "label", "value" and "changeMessage" 
            String attributeNameToDelete = "value"; 
            await sdk.DeleteFieldAttribute(TEST_MODEL_ID, TEST_PASS_ID, fieldKey, attributeNameToDelete);
        }

        static async Task GetPkPassFileExample() {  
            String output = TEST_PASS_ID + ".pkpass";      
            await sdk.GetPkPassFile(TEST_MODEL_ID, TEST_PASS_ID, output);
        }

        static async Task UploadImageExample() { // TOOD        
            await sdk.UploadImage("icon.png");
        }

        static async Task Main(string[] args)
        {
            Console.WriteLine("START");
            await CreatePassExample(); // create a pass with all defaults
            // await CreatePassExample(); // create a pass with custom fields
            // await CreatePassExample2(); // create a with full payload
            // await UpdatePassExample(); // update a pass fully
            // await DeletePassAttributeExample();
            // await DeletePassCustomFieldExample(); 
            // await DeleteFieldAttributeExample(); 
            // await GetPkPassFileExample(); 
            // await UploadImageExample(); 
            Console.WriteLine("END");
        }
    }
}
